<?php

namespace Drupal\files_and_folders\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\node\Entity\Node;
use Drupal\file\Entity\File;
use Drupal\file\FileInterface;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\BinaryFileResponse;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Render\RendererInterface;
use Drupal\Core\File\FileUrlGeneratorInterface;
use Drupal\Core\File\FileSystemInterface;

class FileFolderController extends ControllerBase {

  protected $entityTypeManager;
  protected $logger;
  protected $currentUser;
  protected $renderer;
  protected $fileUrlGenerator;
  

  

  public function __construct(
    EntityTypeManagerInterface $entity_type_manager,
    LoggerChannelFactoryInterface $logger_factory,
    AccountInterface $current_user,
    RendererInterface $renderer,
    FileUrlGeneratorInterface $file_url_generator
  ) {
    $this->entityTypeManager = $entity_type_manager;
    $this->logger = $logger_factory->get('files_and_folders');
    $this->currentUser = $current_user;
    $this->renderer = $renderer;
    $this->fileUrlGenerator = $file_url_generator;
  }

  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager'),
      $container->get('logger.factory'),
      $container->get('current_user'),
      $container->get('renderer'),
      $container->get('file_url_generator')
    );
  }

  public function content() {
    $build = [
      '#theme' => 'files_and_folders_content',
      '#attached' => [
        'library' => ['files_and_folders/files_and_folders'],
      ],
    ];
    return $build;
  }

  public function getFolderTree() {
    try {
      $folders = $this->loadFolders();
      $filtered_folders = $this->filterFoldersByRole($folders);
      $tree = $this->buildTree($filtered_folders);
      return new JsonResponse($tree);
    } catch (\Exception $e) {
      $this->logger->error('Error in getFolderTree: @message', ['@message' => $e->getMessage()]);
      return new JsonResponse(['error' => 'An error occurred while fetching the folder tree.'], 500);
    }
  }

  private function loadFolders() {
    $query = $this->entityTypeManager->getStorage('node')->getQuery()
        ->condition('type', 'folder')
        ->sort('title', 'ASC')
        ->accessCheck(FALSE);  // We'll handle access ourselves
    
    $folder_ids = $query->execute();
    $folders = Node::loadMultiple($folder_ids);
    
    // Filter folders based on role access
    return array_filter($folders, function($folder) {
        return $this->checkFolderAccess($folder);
    });
}

  private function buildTree($folders, $parentId = 0) {
    $branch = [];
    foreach ($folders as $folder) {
      $parent = $folder->get('field_parent_folder')->target_id;
      if ($parent == $parentId) {
        $children = $this->buildTree($folders, $folder->id());
        $folderData = [
          'id' => $folder->id(),
          'name' => $folder->label(),
          'type' => 'folder',
          'hasChildren' => !empty($children),
          'icon_url' => $this->getFolderIconUrl($folder),
          'children' => $children,
        ];
        $branch[] = $folderData;
      }
    }
    return $branch;
  }

  private function getFolderIconUrl($folder) {
    if ($folder->hasField('field_folder_icon') && !$folder->get('field_folder_icon')->isEmpty()) {
      $file = $folder->get('field_folder_icon')->entity;
      if ($file) {
        return \Drupal::service('file_url_generator')->generateAbsoluteString($file->getFileUri());
      }
    }
    return null;
  }

  public function searchContent(Request $request) {
    $searchTerm = $request->request->get('searchTerm');
    $folderId = $request->request->get('folderId');
  
    $query = $this->entityTypeManager->getStorage('node')->getQuery()
      ->accessCheck(TRUE)  // Ajoutez cette ligne
      ->condition('type', ['folder', 'file'], 'IN')
      ->condition('title', '%' . $searchTerm . '%', 'LIKE');
  
    if ($folderId !== '0') {
      $query->condition('field_parent_folder', $folderId);
    }
  
    $ids = $query->execute();
    $items = $this->entityTypeManager->getStorage('node')->loadMultiple($ids);
  
    $filtered_items = $this->filterContentByRole($items);
  
    $content = $this->prepareContentData($filtered_items);
  
    $build = [
      '#theme' => 'files_and_folders_folder_content',
      '#content' => $content,
    ];
  
    $html = $this->renderer->render($build);
  
    return new JsonResponse([
      'html' => $html,
    ]);
  }

 /**
 * Check if user has access to options.
 */
public function checkOptionsAccess() {
  $user = \Drupal::currentUser();
  $hasAccess = $user->hasPermission('access files and folders options') || 
               $user->hasPermission('administer files and folders');
               
  return new JsonResponse([
    'hasAccess' => $hasAccess
  ]);
}
 
  /**
 * Implements hook_node_access().
 */
function files_and_folders_node_access(\Drupal\node\NodeInterface $node, $op, \Drupal\Core\Session\AccountInterface $account) {
  if ($node->getType() === 'folder') {
    // Special handling for root folder
    if (empty($node->get('field_parent_folder')->getValue())) {
      return AccessResult::allowedIfHasPermission($account, 'administer files and folders');
    }
    
    // For other folders
    if ($op === 'update') {
      return AccessResult::allowedIfHasPermission($account, 'edit files and folders');
    }
  }
  return AccessResult::neutral();
}

  private function prepareContentData($items) {
    $content = [];
    foreach ($items as $item) {
      // Base item data
      $itemData = [
        'id' => $item->id(),
        'name' => $item->label(),
        'type' => $item->bundle(),
        'date' => $item->getCreatedTime(),
        'size' => $item->bundle() === 'file' ? $item->get('field_file')->entity->getSize() : '-',
        'visibility' => $item->hasField('field_visibility') ? $item->get('field_visibility')->value : 'undefined',
      ];
  
      // Handle folder icons
      if ($item->bundle() === 'folder') {
        if ($item->hasField('field_folder_icon') && !$item->get('field_folder_icon')->isEmpty()) {
          $file_entity = $item->get('field_folder_icon')->entity;
          if ($file_entity) {
            $itemData['icon_url'] = $this->fileUrlGenerator->generateAbsoluteString($file_entity->getFileUri());
          }
        }
      } 
      // Handle file thumbnails
      else {
        if ($item->hasField('field_thumbnail') && !$item->get('field_thumbnail')->isEmpty()) {
          $thumbnail = $item->get('field_thumbnail')->entity;
          if ($thumbnail) {
            $itemData['thumbnail_url'] = $this->fileUrlGenerator->generateAbsoluteString($thumbnail->getFileUri());
          }
        }
      }
  
      $content[] = $itemData;
    }
    return $content;
  }

  public function getRootFolders() {
    try {
        $query = $this->entityTypeManager->getStorage('node')->getQuery()
            ->condition('type', 'folder')
            ->condition('field_parent_folder', NULL, 'IS NULL')
            ->accessCheck(FALSE);  // We'll handle access ourselves
        
        $folder_ids = $query->execute();
        $folders = $this->entityTypeManager->getStorage('node')->loadMultiple($folder_ids);
        
        // Filter folders using checkFolderAccess
        $filtered_folders = array_filter($folders, function($folder) {
            return $this->checkFolderAccess($folder);
        });

        $content = [];
        foreach ($filtered_folders as $folder) {
            $icon_url = null;
            if ($folder->hasField('field_folder_icon') && !$folder->get('field_folder_icon')->isEmpty()) {
                $file_entity = $folder->get('field_folder_icon')->entity;
                if ($file_entity) {
                    $icon_url = $this->fileUrlGenerator->generateAbsoluteString($file_entity->getFileUri());
                }
            }

            $content[] = [
                'id' => $folder->id(),
                'name' => $folder->label(),
                'type' => 'folder',
                'date' => $folder->getCreatedTime(),
                'size' => '-',
                'visibility' => $folder->hasField('field_visibility') ? $folder->get('field_visibility')->value : 'undefined',
                'icon_url' => $icon_url,
            ];
        }

        $currentLayout = \Drupal::config('files_and_folders.settings')->get('content_layout') ?? 'list';

        $build = [
            '#theme' => 'files_and_folders_folder_content',
            '#content' => $content,
            '#is_root' => TRUE,
            '#layout' => $currentLayout,
        ];

        $html = $this->renderer->render($build);

        return new JsonResponse([
            'html' => $html,
        ]);
    } 
    catch (\Exception $e) {
        $this->logger->error('Error in getRootFolders: @message', ['@message' => $e->getMessage()]);
        return new JsonResponse([
            'error' => 'An error occurred while fetching root folders: ' . $e->getMessage()
        ], 500);
    }
}

public function getFolderContent($folderId) {
  try {
      $folder = Node::load($folderId);
      if (!$folder || $folder->getType() !== 'folder') {
          throw new \Exception("Invalid folder");
      }

      // Check folder access
      if (!$this->checkFolderAccess($folder)) {
          throw new AccessDeniedHttpException('Access denied: You do not have permission to view this folder.');
      }

      $content = $this->loadFolderContent($folder->id());
      $currentLayout = \Drupal::config('files_and_folders.settings')->get('content_layout') ?? 'list';

      $build = [
          '#theme' => 'files_and_folders_folder_content',
          '#folder_name' => $folder->label(),
          '#content' => $content,
          '#layout' => $currentLayout,
      ];

      $html = $this->renderer->render($build);

      return new JsonResponse([
          'folderName' => $folder->label(),
          'html' => $html,
          'layout' => $currentLayout,
      ]);
  } 
  catch (AccessDeniedHttpException $e) {
      $this->logger->warning('Access denied in getFolderContent: @message', [
          '@message' => $e->getMessage()
      ]);
      return new JsonResponse([
          'error' => $e->getMessage()
      ], 403);
  }
  catch (\Exception $e) {
      $this->logger->error('Error in getFolderContent: @message', [
          '@message' => $e->getMessage()
      ]);
      return new JsonResponse([
          'error' => 'An error occurred while fetching folder content.'
      ], 500);
  }
    catch (AccessDeniedHttpException $e) {
        $this->logger->warning('Access denied in getFolderContent: @message', [
            '@message' => $e->getMessage()
        ]);
        return new JsonResponse([
            'error' => $e->getMessage()
        ], 403);
    }
    catch (\Exception $e) {
        $this->logger->error('Error in getFolderContent: @message', [
            '@message' => $e->getMessage()
        ]);
        return new JsonResponse([
            'error' => 'An error occurred while fetching folder content.'
        ], 500);
    }
}

private function loadFolderContent($folderId) {
  // First check if user has access to this folder
  $folder = Node::load($folderId);
  if (!$folder || !$this->checkFolderAccess($folder)) {
      return [];
  }

  $query = $this->entityTypeManager->getStorage('node')->getQuery()
      ->condition('type', ['file', 'folder'], 'IN')
      ->condition('field_parent_folder', $folderId)
      ->accessCheck(FALSE);  // We handle access control ourselves
  
  $content_ids = $query->execute();
  $content = Node::loadMultiple($content_ids);
  $items = [];
  
  foreach ($content as $item) {
      // Only include items if user has proper access
      if ($this->checkFolderAccess($folder)) {  // Check parent folder access
          $itemData = [
              'type' => $item->bundle(),
              'name' => $item->getTitle(),
              'date' => \Drupal::service('date.formatter')->format($item->getCreatedTime(), 'custom', 'F j, Y - g:ia'),
              'visibility' => $item->get('field_visibility')->value,
              'id' => $item->id(),
              'parent_id' => $folderId,  // Add parent ID for access checking
          ];

          // Add folder/file specific data
          if ($item->bundle() === 'folder') {
              $itemData['size'] = $this->calculateFolderSize($item->id());
              $itemData['hasChildren'] = $this->folderHasChildren($item->id());
              if ($item->hasField('field_folder_icon') && !$item->get('field_folder_icon')->isEmpty()) {
                  $file_entity = $item->get('field_folder_icon')->entity;
                  if ($file_entity) {
                      $itemData['icon_url'] = $this->fileUrlGenerator->generateAbsoluteString($file_entity->getFileUri());
                  }
              }
          } else {
              // File handling
              $file = $item->get('field_file')->entity;
              if ($file) {
                  $itemData['size'] = $file->getSize();
                  $itemData['uri'] = $file->getFileUri();
                  $itemData['file_url'] = $this->fileUrlGenerator->generateAbsoluteString($file->getFileUri());
              }

              if ($item->hasField('field_thumbnail') && !$item->get('field_thumbnail')->isEmpty()) {
                  $thumbnail = $item->get('field_thumbnail')->entity;
                  if ($thumbnail) {
                      $itemData['thumbnail_url'] = $this->fileUrlGenerator->generateAbsoluteString($thumbnail->getFileUri());
                  }
              }
          }
          
          $items[] = $itemData;
      }
  }
  
  return $items;
}

private function checkFolderAccess($folder) {
  if (!$folder) {
      return false;
  }

  // Get authorized roles for the folder
  if (!$folder->hasField('field_authorized_roles') || $folder->get('field_authorized_roles')->isEmpty()) {
      return false;
  }

  // Get the authorized roles for this folder
  $authorized_roles = array_map(function($role) {
      return $role->id();
  }, $folder->get('field_authorized_roles')->referencedEntities());

  // Get user roles (excluding 'authenticated')
  $user_roles = array_diff($this->currentUser->getRoles(), ['authenticated']);

  // Check if user has any of the authorized roles
  $has_access = !empty(array_intersect($authorized_roles, $user_roles));

  // Log access check for debugging
  $this->logger->notice('Folder access check for folder @folder_id: User roles: @user_roles, Authorized roles: @auth_roles, Access: @access', [
      '@folder_id' => $folder->id(),
      '@user_roles' => implode(', ', $user_roles),
      '@auth_roles' => implode(', ', $authorized_roles),
      '@access' => $has_access ? 'granted' : 'denied'
  ]);

  return $has_access;
}
  private function filterFoldersByRole($folders) {
    return array_filter($folders, [$this, 'checkFolderAccess']);
  }

  private function filterContentByRole($content) {
    return array_filter($content, function($item) {
        // For Node objects
        if ($item instanceof Node) {
            if ($item->bundle() === 'folder') {
                return $this->checkFolderAccess($item);
            } else {
                // For files, check parent folder access
                $parentId = $item->get('field_parent_folder')->target_id;
                if ($parentId) {
                    $parentFolder = Node::load($parentId);
                    return $parentFolder ? $this->checkFolderAccess($parentFolder) : false;
                }
            }
        }
        // For array data
        elseif (is_array($item)) {
            if ($item['type'] === 'folder') {
                $node = Node::load($item['id']);
                return $node ? $this->checkFolderAccess($node) : false;
            } else {
                // For files in array format, check parent folder access
                $parentId = $item['parent_id'] ?? null;
                if ($parentId) {
                    $parentFolder = Node::load($parentId);
                    return $parentFolder ? $this->checkFolderAccess($parentFolder) : false;
                }
            }
        }
        return false;
    });
}

  private function calculateFolderSize($folderId) {
    $totalSize = 0;
    $query = $this->entityTypeManager->getStorage('node')->getQuery()
      ->condition('type', ['file', 'folder'], 'IN')
      ->condition('field_parent_folder', $folderId)
      ->accessCheck(TRUE);
    $contentIds = $query->execute();
    $content = Node::loadMultiple($contentIds);
    foreach ($content as $item) {
      if ($item->bundle() == 'file') {
        $totalSize += $item->get('field_file_size')->value;
      } elseif ($item->bundle() == 'folder') {
        $totalSize += $this->calculateFolderSize($item->id());
      }
    }
    return $totalSize;
  }

private function formatSize($bytes) {
  $bytes = max($bytes, 0);
  $size_mb = $bytes / (1024 * 1024);
  return number_format($size_mb, 2, '.', '') . ' MB';
}

private function folderHasChildren($folderId) {
  $query = $this->entityTypeManager->getStorage('node')->getQuery()
    ->condition('type', ['file', 'folder'], 'IN')
    ->condition('field_parent_folder', $folderId)
    ->accessCheck(TRUE)
    ->range(0, 1);
  return !empty($query->execute());
}


public function updateItemName(Request $request) {
  try {
    $item_id = $request->request->get('item_id');
    $new_name = $request->request->get('new_name');
    $item_type = $request->request->get('item_type');

    $item = Node::load($item_id);
    if (!$item) {
      throw new \Exception('Item not found');
    }

    // Check access
    if (!$item->access('update')) {
      throw new AccessDeniedHttpException();
    }

    $item->setTitle($new_name);
    $item->save();

    $parent_id = $item->get('field_parent_folder')->target_id;

    return new JsonResponse([
      'status' => 'success',
      'message' => 'Item name updated successfully',
      'parent_id' => $parent_id,
    ]);
  } catch (\Exception $e) {
    return new JsonResponse([
      'status' => 'error',
      'message' => $e->getMessage()
    ], 500);
  }
}

public function downloadFile($fileId) {
  try {
    $node = Node::load($fileId);
    if (!$node || $node->getType() !== 'file') {
      throw new NotFoundHttpException('File not found');
    }

    // Check access
    if (!$node->access('view')) {
      throw new AccessDeniedHttpException();
    }

    $file_field = $node->get('field_file')->entity;
    if (!$file_field) {
      throw new NotFoundHttpException('File not found');
    }

    $uri = $file_field->getFileUri();
    $filename = $file_field->getFilename();

    $headers = [
      'Content-Type' => $file_field->getMimeType(),
      'Content-Disposition' => 'attachment; filename="' . $filename . '"',
      'Content-Length' => $file_field->getSize(),
    ];

    return new BinaryFileResponse($uri, 200, $headers, true);
  } catch (\Exception $e) {
    $this->loggerFactory->get('files_and_folders')->error('Error downloading file: @message', ['@message' => $e->getMessage()]);
    throw new NotFoundHttpException('Error downloading file');
  }
}

public function moveItem(Request $request) {
  try {
    $item_id = $request->request->get('item_id');
    $new_parent_id = $request->request->get('new_parent_id');
    $item_type = $request->request->get('item_type');

    $item = Node::load($item_id);
    if (!$item) {
      throw new \Exception('Item not found');
    }

    // Check access
    if (!$item->access('update')) {
      throw new AccessDeniedHttpException();
    }

    $item->set('field_parent_folder', $new_parent_id);
    $item->save();

    return new JsonResponse([
      'status' => 'success',
      'message' => 'Item moved successfully'
    ]);
  } catch (\Exception $e) {
    return new JsonResponse([
      'status' => 'error',
      'message' => $e->getMessage()
    ], 500);
  }
}

public function deleteItem(Request $request) {
  try {
    $item_id = $request->request->get('item_id');
    $item_type = $request->request->get('item_type');

    $node = Node::load($item_id);
    if (!$node) {
      throw new NotFoundHttpException('Item not found');
    }

    // Check access
    if (!$node->access('delete')) {
      throw new AccessDeniedHttpException();
    }

    // Store parent ID before deletion
    $parent_id = $node->get('field_parent_folder')->target_id;

    // Check if it's a folder and handle icon deletion
    if ($node->bundle() === 'folder') {
      if ($node->hasField('field_folder_icon') && !$node->get('field_folder_icon')->isEmpty()) {
        $icon = $node->get('field_folder_icon')->entity;
        if ($icon) {
          // Get the icon path
          $uri = $icon->getFileUri();
          $realPath = \Drupal::service('file_system')->realpath($uri);
          
          // Delete icon from filesystem
          if ($realPath && file_exists($realPath)) {
            unlink($realPath);
            $this->logger->notice('Folder icon deleted: @path', [
              '@path' => $realPath
            ]);
          }
          
          // Delete the file entity
          $icon->delete();
        }
      }
    }
    // Check if it's a file
    elseif ($node->bundle() === 'file') {
      // Delete the main file
      if ($node->hasField('field_file') && !$node->get('field_file')->isEmpty()) {
        $file = $node->get('field_file')->entity;
        if ($file) {
          // Get the file path
          $uri = $file->getFileUri();
          $realPath = \Drupal::service('file_system')->realpath($uri);
          
          // Delete file from filesystem
          if ($realPath && file_exists($realPath)) {
            unlink($realPath);
          }
          
          // Delete the file entity
          $file->delete();
        }
      }

      // Delete thumbnail if exists
      if ($node->hasField('field_thumbnail') && !$node->get('field_thumbnail')->isEmpty()) {
        $thumbnail = $node->get('field_thumbnail')->entity;
        if ($thumbnail) {
          // Get thumbnail path
          $thumbnailUri = $thumbnail->getFileUri();
          $thumbnailPath = \Drupal::service('file_system')->realpath($thumbnailUri);
          
          // Delete thumbnail file from filesystem
          if ($thumbnailPath && file_exists($thumbnailPath)) {
            unlink($thumbnailPath);
            $this->logger->notice('Thumbnail deleted: @path', [
              '@path' => $thumbnailPath
            ]);
          }
          
          // Delete thumbnail entity
          $thumbnail->delete();
        }
      }
    }

    // Delete the node
    $node->delete();

    // Log successful deletion
    $this->logger->notice('Item @id deleted successfully', [
      '@id' => $item_id
    ]);

    return new JsonResponse([
      'status' => 'success',
      'message' => t('Item deleted successfully'),
      'parent_id' => $parent_id,
    ]);
  }
  catch (\Exception $e) {
    $this->logger->error('Error deleting item: @message', [
      '@message' => $e->getMessage()
    ]);
    
    return new JsonResponse([
      'status' => 'error',
      'message' => t('Error deleting item: @message', ['@message' => $e->getMessage()])
    ], 500);
  }
}

public function uploadFiles(Request $request) {
  try {
    $config = $this->config('files_and_folders.settings');
    $file_system = $config->get('file_system') ?: 'public';
    $custom_directory = $config->get('custom_directory') ?: 'files_and_folders';

    $parentId = $request->request->get('parent_id');
    $visibility = $request->request->get('visibility');
    $uploadedFiles = $request->files->get('files');

    if (!$uploadedFiles) {
      throw new \Exception('No files were uploaded.');
    }

    // Get parent folder's authorized roles
    $authorizedRoles = [];
    if ($parentId) {
      $parentNode = Node::load($parentId);
      if ($parentNode && $parentNode->hasField('field_authorized_roles')) {
        $authorizedRoles = $parentNode->get('field_authorized_roles')->getValue();
      }
    }

    $results = [];
    foreach ($uploadedFiles as $uploadedFile) {
      $fileName = $uploadedFile->getClientOriginalName();
      $fileSize = $uploadedFile->getSize();
      $mimeType = $uploadedFile->getMimeType();

      // Log file info
      $this->logger->notice('Uploading file: @name, MIME: @mime', [
        '@name' => $fileName,
        '@mime' => $mimeType,
      ]);

      // Prepare directory and save file
      $directory = $file_system . '://' . $custom_directory;
      \Drupal::service('file_system')->prepareDirectory($directory, \Drupal\Core\File\FileSystemInterface::CREATE_DIRECTORY);

      $file = File::create([
        'uri' => $directory . '/' . $fileName,
        'filename' => $fileName,
        'filemime' => $mimeType,
        'status' => \Drupal\file\FileInterface::STATUS_PERMANENT,
      ]);
      $file->save();

      \Drupal::service('file_system')->move(
        $uploadedFile->getRealPath(),
        $file->getFileUri()
      );

      // Determine file type using both MIME type and filename
      $fileType = $this->determineFileType($mimeType, $fileName);

      // Log file type detection
      $this->logger->notice('File type detected: @type for file: @file', [
        '@type' => print_r($fileType, true),
        '@file' => $fileName,
      ]);

      // Create node with basic fields
      $node = Node::create([
        'type' => 'file',
        'title' => $fileName,
        'field_file' => [
          'target_id' => $file->id(),
          'display' => 1,
          'description' => '',
        ],
        'field_parent_folder' => ['target_id' => $parentId],
        'field_file_size' => $fileSize,
        'field_visibility' => ['value' => $visibility],
        'field_file_type' => $fileType,
        'field_authorized_roles' => $authorizedRoles,
      ]);

      // Generate thumbnail based on file type
      if ($fileType['value'] === 'image') {
        $this->logger->notice('Generating thumbnail for image: @name', [
          '@name' => $fileName
        ]);
      
        // Create thumbnail directory
        $thumbnailDirectory = 'public://files_and_folders/thumbnails/' . date('Y-m');
        \Drupal::service('file_system')->prepareDirectory($thumbnailDirectory, FileSystemInterface::CREATE_DIRECTORY);
      
        // Generate thumbnail using the service
        $image = \Drupal::service('image.factory')->get($file->getFileUri());
        if ($image->isValid()) {
          $thumbnailUri = $thumbnailDirectory . '/thumbnail_' . $fileName;
          $image->scale(300, 300);
          $image->save($thumbnailUri);
      
          if (file_exists($thumbnailUri)) {
            $thumbnail_file = File::create([
              'uri' => $thumbnailUri,
              'filename' => 'thumbnail_' . $fileName,
              'filemime' => 'image/jpeg',
              'status' => \Drupal\file\FileInterface::STATUS_PERMANENT,
            ]);
            $thumbnail_file->save();
      
            $node->set('field_thumbnail', [
              'target_id' => $thumbnail_file->id(),
              'alt' => t('Thumbnail for @filename', ['@filename' => $fileName]),
            ]);
      
            $this->logger->notice('Thumbnail created: @uri', [
              '@uri' => $thumbnailUri
            ]);
          }
        }
      }
      elseif ($fileType['value'] === 'svg') {
        // Skip thumbnail generation for SVG files
        $this->logger->notice('SVG file uploaded, skipping thumbnail generation: @name', [
          '@name' => $fileName
        ]);
        
        // Set the SVG file itself as the thumbnail
        $node->set('field_thumbnail', [
          'target_id' => $file->id(),
          'alt' => t('SVG @filename', ['@filename' => $fileName]),
        ]);
      }
      elseif ($fileType['value'] === 'pdf') {
        $this->logger->notice('Generating thumbnail for PDF: @name', [
          '@name' => $fileName
        ]);
      
        try {
          // Create thumbnail directory
          $thumbnailDirectory = 'public://thumbnails/' . date('Y-m');
          \Drupal::service('file_system')->prepareDirectory($thumbnailDirectory, FileSystemInterface::CREATE_DIRECTORY);
      
          // Get real path of PDF file
          $pdfPath = \Drupal::service('file_system')->realpath($file->getFileUri());
          
          if (!$pdfPath || !file_exists($pdfPath)) {
            throw new \Exception('PDF file not found at: ' . $pdfPath);
          }
      
          // Create Imagick instance
          $imagick = new \Imagick();
          
          // Set resolution for better quality
          $imagick->setResolution(300, 300);
          
          // Read only the first page of the PDF
          $imagick->readImage($pdfPath . '[0]');
          
          // Convert to JPG
          $imagick->setImageFormat('jpeg');
          
          // Set compression and quality
          $imagick->setImageCompression(\Imagick::COMPRESSION_JPEG);
          $imagick->setImageCompressionQuality(90);
          
          // Resize to thumbnail size (maintaining aspect ratio)
          $imagick->thumbnailImage(300, 300, true);
      
          // Set white background (for transparency)
          $imagick->setImageBackgroundColor('white');
          $imagick->setImageAlphaChannel(\Imagick::ALPHACHANNEL_REMOVE);
          
          // Create thumbnail path
          $thumbnailUri = $thumbnailDirectory . '/thumbnail_' . pathinfo($fileName, PATHINFO_FILENAME) . '.jpg';
          $thumbnailPath = \Drupal::service('file_system')->realpath($thumbnailUri);
          
          // Save the thumbnail
          $imagick->writeImage($thumbnailPath);
          $imagick->clear();
          $imagick->destroy();
      
          if (file_exists($thumbnailPath)) {
            // Create thumbnail file entity
            $thumbnail_file = File::create([
              'uri' => $thumbnailUri,
              'filename' => 'thumbnail_' . $fileName . '.jpg',
              'filemime' => 'image/jpeg',
              'status' => \Drupal\file\FileInterface::STATUS_PERMANENT,
            ]);
            $thumbnail_file->save();
      
            // Set the thumbnail field
            $node->set('field_thumbnail', [
              'target_id' => $thumbnail_file->id(),
              'alt' => t('Thumbnail for @filename', ['@filename' => $fileName]),
            ]);
      
            $this->logger->notice('PDF thumbnail created: @uri', [
              '@uri' => $thumbnailUri
            ]);
          }
        }
        catch (\Exception $e) {
          $this->logger->error('Error generating PDF thumbnail: @message', [
            '@message' => $e->getMessage()
          ]);
        }
      }
      
      elseif ($fileType['value'] === 'document' && in_array(strtolower(pathinfo($fileName, PATHINFO_EXTENSION)), ['txt'])) {
        $this->logger->notice('Generating thumbnail for text file: @name', [
          '@name' => $fileName
        ]);
      
        try {
          // Create thumbnail directory
          $thumbnailDirectory = 'public://thumbnails/' . date('Y-m');
          \Drupal::service('file_system')->prepareDirectory($thumbnailDirectory, FileSystemInterface::CREATE_DIRECTORY);
      
          // Get real path of the file
          $filePath = \Drupal::service('file_system')->realpath($file->getFileUri());
          
          if (!$filePath || !file_exists($filePath)) {
            throw new \Exception('Text file not found at: ' . $filePath);
          }
      
          // Create Imagick instance
          $imagick = new \Imagick();
          
          // Create a new image with white background
          $imagick->newImage(300, 300, 'white');
          
          // Set up the drawing properties
          $draw = new \ImagickDraw();
          $draw->setFontSize(12);
          $draw->setFillColor('black');
          
          // Read first few lines of text file
          $content = file_get_contents($filePath, FALSE, NULL, 0, 500); // Read first 500 characters
          $lines = explode("\n", wordwrap($content, 40)); // Wrap text at 40 characters
          
          // Only take first 15 lines
          $lines = array_slice($lines, 0, 15);
          
          // Add text to image
          $y = 30;
          foreach ($lines as $line) {
            $imagick->annotateImage($draw, 20, $y, 0, $line);
            $y += 18; // Line spacing
          }
          
          // Add file name at the bottom
          $draw->setFontSize(14);
          $draw->setFillColor('blue');
          $imagick->annotateImage($draw, 20, 280, 0, basename($fileName));
          
          // Set format to JPEG
          $imagick->setImageFormat('jpeg');
          
          // Set compression and quality
          $imagick->setImageCompression(\Imagick::COMPRESSION_JPEG);
          $imagick->setImageCompressionQuality(90);
          
          // Create thumbnail path
          $thumbnailUri = $thumbnailDirectory . '/thumbnail_' . pathinfo($fileName, PATHINFO_FILENAME) . '.jpg';
          $thumbnailPath = \Drupal::service('file_system')->realpath($thumbnailUri);
          
          // Save the thumbnail
          $imagick->writeImage($thumbnailPath);
          $imagick->clear();
          $imagick->destroy();
      
          if (file_exists($thumbnailPath)) {
            // Create thumbnail file entity
            $thumbnail_file = File::create([
              'uri' => $thumbnailUri,
              'filename' => 'thumbnail_' . $fileName . '.jpg',
              'filemime' => 'image/jpeg',
              'status' => \Drupal\file\FileInterface::STATUS_PERMANENT,
            ]);
            $thumbnail_file->save();
      
            // Set the thumbnail field
            $node->set('field_thumbnail', [
              'target_id' => $thumbnail_file->id(),
              'alt' => t('Thumbnail for @filename', ['@filename' => $fileName]),
            ]);
      
            $this->logger->notice('Text file thumbnail created: @uri', [
              '@uri' => $thumbnailUri
            ]);
          }
        }
        catch (\Exception $e) {
          $this->logger->error('Error generating text file thumbnail: @message', [
            '@message' => $e->getMessage()
          ]);
        }
      }
      
      elseif ($fileType['value'] === 'spreadsheet' || $fileType['value'] === 'document' || $fileType['value'] === 'presentation') {
        $this->logger->notice('Generating thumbnail for office document: @name', [
          '@name' => $fileName
        ]);
      
        try {
          // Create thumbnail directory
          $thumbnailDirectory = 'public://thumbnails/' . date('Y-m');
          \Drupal::service('file_system')->prepareDirectory($thumbnailDirectory, FileSystemInterface::CREATE_DIRECTORY);
      
          // Get real path of the file
          $filePath = \Drupal::service('file_system')->realpath($file->getFileUri());
          
          if (!$filePath || !file_exists($filePath)) {
            throw new \Exception('Office document file not found at: ' . $filePath);
          }
      
          // Create temporary directory for PDF conversion
          $tempDir = \Drupal::service('file_system')->getTempDirectory() . '/office_thumb_' . uniqid();
          \Drupal::service('file_system')->prepareDirectory($tempDir, FileSystemInterface::CREATE_DIRECTORY | FileSystemInterface::MODIFY_PERMISSIONS);
          
          // Set proper permissions for temp directory
          chmod($tempDir, 0777);
      
          // Set HOME environment variable for LibreOffice
          putenv('HOME=/var/www');
      
          // Convert Office document to PDF using LibreOffice with user directory specified
          $command = sprintf(
            'HOME=/var/www soffice --headless --convert-to pdf --outdir %s %s 2>&1',
            escapeshellarg($tempDir),
            escapeshellarg($filePath)
          );
          
          // Log the command
          $this->logger->notice('Executing command: @command', ['@command' => $command]);
          
          // Execute command and capture output
          exec($command, $output, $return_var);
          
          // Log the output
          $this->logger->notice('Command output: @output', ['@output' => implode("\n", $output)]);
      
          if ($return_var !== 0) {
            throw new \Exception('Failed to convert office document to PDF. Return code: ' . $return_var);
          }
      
          // Get the generated PDF file
          $pdfFiles = glob($tempDir . '/*.pdf');
          if (empty($pdfFiles)) {
            throw new \Exception('No PDF file generated in: ' . $tempDir);
          }
      
          // Create Imagick instance
          $imagick = new \Imagick();
          
          // Set resolution for better quality
          $imagick->setResolution(300, 300);
          
          // Read the first page of the converted PDF
          $imagick->readImage($pdfFiles[0] . '[0]');
          
          // Convert to JPG
          $imagick->setImageFormat('jpeg');
          
          // Set compression and quality
          $imagick->setImageCompression(\Imagick::COMPRESSION_JPEG);
          $imagick->setImageCompressionQuality(90);
          
          // Resize to thumbnail size (maintaining aspect ratio)
          $imagick->thumbnailImage(300, 300, true);
      
          // Set white background
          $imagick->setImageBackgroundColor('white');
          $imagick->setImageAlphaChannel(\Imagick::ALPHACHANNEL_REMOVE);
          
          // Create thumbnail path
          $thumbnailUri = $thumbnailDirectory . '/thumbnail_' . pathinfo($fileName, PATHINFO_FILENAME) . '.jpg';
          $thumbnailPath = \Drupal::service('file_system')->realpath($thumbnailUri);
          
          // Save the thumbnail
          $imagick->writeImage($thumbnailPath);
          $imagick->clear();
          $imagick->destroy();
      
          // Clean up temporary files
          \Drupal::service('file_system')->deleteRecursive($tempDir);
      
          if (file_exists($thumbnailPath)) {
            // Create thumbnail file entity
            $thumbnail_file = File::create([
              'uri' => $thumbnailUri,
              'filename' => 'thumbnail_' . $fileName . '.jpg',
              'filemime' => 'image/jpeg',
              'status' => \Drupal\file\FileInterface::STATUS_PERMANENT,
            ]);
            $thumbnail_file->save();
      
            // Set the thumbnail field
            $node->set('field_thumbnail', [
              'target_id' => $thumbnail_file->id(),
              'alt' => t('Thumbnail for @filename', ['@filename' => $fileName]),
            ]);
      
            $this->logger->notice('Office document thumbnail created: @uri', [
              '@uri' => $thumbnailUri
            ]);
          }
        }
        catch (\Exception $e) {
          $this->logger->error('Error generating office document thumbnail: @message', [
            '@message' => $e->getMessage()
          ]);
        }
      }

      $node->save();

      $results[] = [
        'name' => $fileName,
        'id' => $node->id(),
        'size' => $this->formatSizeInBytes($fileSize),
        'visibility' => $visibility,
        'type' => $fileType['value'],
      ];
    }

    return new JsonResponse([
      'status' => 'success',
      'message' => 'Files uploaded successfully',
      'files' => $results
    ]);
  } 
  catch (\Exception $e) {
    $this->logger->error('Error uploading files: @message', ['@message' => $e->getMessage()]);
    return new JsonResponse(['status' => 'error', 'message' => $e->getMessage()], 500);
  }
}

private function checkLibreOffice() {
  exec('which soffice 2>&1', $output, $return_var);
  $this->logger->notice('LibreOffice check: @output', ['@output' => implode("\n", $output)]);
  return $return_var === 0;
}

/**
 * Determines the file type based on MIME type and file extension.
 */
private function determineFileType($mimeType, $fileName = '') {
  // Log the MIME type and filename for debugging
  $this->logger->notice('Detecting file type - MIME: @mime_type, File: @file', [
    '@mime_type' => $mimeType,
    '@file' => $fileName,
  ]);

  // Get file extension if filename is provided
  $extension = strtolower(pathinfo($fileName, PATHINFO_EXTENSION));

  // SVG check first
  if ($mimeType === 'image/svg+xml' || $extension === 'svg') {
    $this->logger->notice('File type detected as SVG');
    return ['value' => 'svg'];
  }

  // Image types by MIME
  $imageTypes = [
    'image/jpeg',
    'image/png',
    'image/gif',
    'image/webp',
    'image/bmp',
    'image/tiff',
  ];

  // Image extensions
  $imageExtensions = ['jpg', 'jpeg', 'png', 'gif', 'webp', 'bmp', 'tiff'];

  // If MIME type is not specific (application/octet-stream), use file extension
  if ($mimeType === 'application/octet-stream' && !empty($extension)) {
    if (in_array($extension, $imageExtensions)) {
      $this->logger->notice('File type detected as image from extension: @ext', ['@ext' => $extension]);
      return ['value' => 'image'];
    }
    if ($extension === 'pdf') {
      return ['value' => 'pdf'];
    }
    if (in_array($extension, ['xls', 'xlsx', 'csv'])) {
      return ['value' => 'spreadsheet'];
    }
    if (in_array($extension, ['ppt', 'pptx'])) {
      return ['value' => 'presentation'];
    }
    if (in_array($extension, ['doc', 'docx', 'txt', 'rtf'])) {
      return ['value' => 'document'];
    }
  }

  // Check MIME type first
  if (in_array($mimeType, $imageTypes)) {
    $this->logger->notice('File type detected as image from MIME type');
    return ['value' => 'image'];
  }

  // PDF types
  $pdfTypes = ['application/pdf'];
  if (in_array($mimeType, $pdfTypes)) {
    return ['value' => 'pdf'];
  }

  // Spreadsheet types
  $spreadsheetTypes = [
    'application/vnd.ms-excel',
    'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet',
    'application/vnd.oasis.opendocument.spreadsheet',
    'text/csv',
  ];
  if (in_array($mimeType, $spreadsheetTypes)) {
    return ['value' => 'spreadsheet'];
  }

  // Presentation types
  $presentationTypes = [
    'application/vnd.ms-powerpoint',
    'application/vnd.openxmlformats-officedocument.presentationml.presentation',
    'application/vnd.oasis.opendocument.presentation',
  ];
  if (in_array($mimeType, $presentationTypes)) {
    return ['value' => 'presentation'];
  }

  // Document types
  $documentTypes = [
    'application/msword',
    'application/vnd.openxmlformats-officedocument.wordprocessingml.document',
    'application/vnd.oasis.opendocument.text',
    'text/plain',
    'text/rtf',
  ];
  if (in_array($mimeType, $documentTypes)) {
    return ['value' => 'document'];
  }

  // Log unrecognized MIME type
  $this->logger->notice('Unrecognized MIME type defaulting to document: @mime_type', [
    '@mime_type' => $mimeType
  ]);

  return ['value' => 'document'];
}


public function getFileUrl($fileId) {
  try {
    $file = File::load($fileId);
    if (!$file) {
      throw new NotFoundHttpException();
    }

    // Get the base URL of your site
    $base_url = \Drupal::request()->getSchemeAndHttpHost();
    
    // Get file URI and convert to URL
    $uri = $file->getFileUri();
    $path = str_replace('public://', '/sites/default/files/', $uri);
    $url = $base_url . $path;

    // Get file extension
    $extension = strtolower(pathinfo($file->getFilename(), PATHINFO_EXTENSION));
    
    // List of text file extensions including CSV
    $textExtensions = ['txt', 'json', 'csv'];
    
    if (in_array($extension, $textExtensions)) {
      // For text files, set Content-Type header
      $mimeTypes = [
        'txt' => 'text/plain',
        'json' => 'application/json',
        'csv' => 'text/csv',
      ];
      
      $headers = [
        'Content-Type' => $mimeTypes[$extension] ?? 'text/plain',
        'Content-Disposition' => 'inline; filename="' . $file->getFilename() . '"'
      ];
      
      // For CSV files, add specific headers for better browser handling
      if ($extension === 'csv') {
        $headers['Content-Type'] = 'text/csv; charset=utf-8';
      }
      
      return new BinaryFileResponse($file->getFileUri(), 200, $headers, true);
    }

    // Log the generated URL for debugging
    \Drupal::logger('files_and_folders')->notice('Generated file URL: @url', ['@url' => $url]);
    
    return new JsonResponse(['url' => $url]);
  }
  catch (\Exception $e) {
    \Drupal::logger('files_and_folders')->error('Error getting file URL: @message', ['@message' => $e->getMessage()]);
    return new JsonResponse(['error' => $e->getMessage()], 500);
  }
}

private function formatSizeInMB($bytes) {
  $sizeMB = $bytes / (1024 * 1024);
  if ($sizeMB == 0) {
      return '0';
  } else {
      return number_format($sizeMB, 2, '.', '') . ' MB';
  }
}

private function formatSizeInBytes($bytes, $isFolder = false) {
  if ($isFolder && $bytes == 0) {
      return '-';
  } elseif ($bytes == 0) {
      return '0 B';
  } elseif ($bytes < 1024) {
      return $bytes . ' B';
  } elseif ($bytes < 1048576) {
      return round($bytes / 1024, 2) . ' KB';
  } else {
      return round($bytes / 1048576, 2) . ' MB';
  }
}


public function saveLayout(Request $request) {
  try {
    $layout = $request->request->get('layout');
    \Drupal::configFactory()->getEditable('files_and_folders.settings')
      ->set('content_layout', $layout)
      ->save();
    return new JsonResponse(['status' => 'success']);
  } catch (\Exception $e) {
    return new JsonResponse(['status' => 'error', 'message' => $e->getMessage()], 500);
  }
}
public function getLayout() {
  $layout = \Drupal::config('files_and_folders.settings')->get('content_layout') ?? 'list';
  return new JsonResponse(['layout' => $layout]);
}

public function getRoles() {
  $roles = user_roles(TRUE);
  $roleOptions = [];
  foreach ($roles as $role_id => $role) {
    $roleOptions[] = [
      'id' => $role_id,
      'label' => $role->label(),
    ];
  }
  return new JsonResponse($roleOptions);
}

public function createRootFolder(Request $request) {
  try {
    $folder_name = $request->request->get('name');
    $authorized_roles = json_decode($request->request->get('allowed_roles'), TRUE);
    
    $node = Node::create([
      'type' => 'folder',
      'title' => $folder_name,
      'field_parent_folder' => null,
      'field_authorized_roles' => $authorized_roles,
    ]);

    // Handle icon upload
    if ($request->files->has('icon')) {
      $icon_file = $request->files->get('icon');
      
      // Ensure the upload directory exists
      $directory = 'public://files_and_folders/folder-icons/';
      \Drupal::service('file_system')->prepareDirectory($directory, FileSystemInterface::CREATE_DIRECTORY);

      // Save the uploaded file
      $file = File::create([
        'uri' => $directory . $icon_file->getClientOriginalName(),
        'filename' => $icon_file->getClientOriginalName(),
        'status' => FileInterface::STATUS_PERMANENT,
      ]);
      
      // Move the uploaded file to the specified directory
      \Drupal::service('file_system')->moveUploadedFile($icon_file->getRealPath(), $file->getFileUri());
      
      $file->save();

      // Set the icon for the folder
      $node->set('field_folder_icon', [
        'target_id' => $file->id(),
        'alt' => $folder_name . ' icon',
        'title' => $folder_name . ' icon',
      ]);
    }

    $node->save();

    return new JsonResponse([
      'status' => 'success',
      'message' => 'Root folder created successfully'
    ]);
  } catch (\Exception $e) {
    \Drupal::logger('files_and_folders')->error('Error creating root folder: @message', ['@message' => $e->getMessage()]);
    return new JsonResponse([
      'status' => 'error',
      'message' => $e->getMessage()
    ], 500);
  }
}

public function saveIconSize(Request $request) {
  try {
    $size = $request->request->get('size');
    // Validate the size
    if (!in_array($size, ['small', 'medium', 'large'])) {
      throw new \Exception('Invalid icon size');
    }
    // Save the size to configuration
    \Drupal::configFactory()->getEditable('files_and_folders.settings')
      ->set('icon_size', $size)
      ->save();

    return new JsonResponse(['status' => 'success', 'message' => 'Icon size saved successfully']);
  } catch (\Exception $e) {
    return new JsonResponse(['status' => 'error', 'message' => $e->getMessage()], 500);
  }
}

public function getIconSize() {
  $size = \Drupal::config('files_and_folders.settings')->get('icon_size') ?? 'medium';
  return new JsonResponse(['size' => $size]);
}

public function createFolder(Request $request) {
  try {
    $name = $request->request->get('name');
    $parent_id = $request->request->get('parent_id');
    $visibility = $request->request->get('visibility');

    // Validate input
    if (empty($name) || empty($visibility)) {
      throw new \Exception('Folder name and visibility are required.');
    }

    // Check for existing folder with the same name
    $query = $this->entityTypeManager->getStorage('node')->getQuery()
      ->condition('type', 'folder')
      ->condition('title', $name)
      ->condition('field_parent_folder', $parent_id)
      ->accessCheck(TRUE);
    
    $existing = $query->execute();

    if (!empty($existing)) {
      return new JsonResponse(['status' => 'error', 'message' => 'A folder with this name already exists in the selected location.'], 400);
    }

    // Load the parent folder node
    $parent_folder = Node::load($parent_id);

    // Get the authorized roles from the parent folder
    $authorized_roles = [];
    if ($parent_folder && $parent_folder->hasField('field_authorized_roles')) {
      $authorized_roles = $parent_folder->get('field_authorized_roles')->getValue();
    }

    // Create the new folder node
    $folder = Node::create([
      'type' => 'folder',
      'title' => $name,
      'field_parent_folder' => $parent_id,
      'field_visibility' => $visibility,
      'field_authorized_roles' => $authorized_roles,
    ]);
    $folder->save();

    return new JsonResponse(['status' => 'success', 'message' => 'Folder created successfully', 'id' => $folder->id()]);
  } catch (\Exception $e) {
    $this->loggerFactory->get('files_and_folders')->error('Error creating folder: @message', ['@message' => $e->getMessage()]);
    return new JsonResponse(['status' => 'error', 'message' => 'An error occurred while creating the folder.'], 500);
  }
}
}