<?php

namespace Drupal\files_and_folders\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\node\Entity\Node;
use Drupal\user\Entity\Role;
use Drupal\file\Entity\File;

class FilesAndFoldersSettingsForm extends ConfigFormBase {

  protected function getEditableConfigNames() {
    return ['files_and_folders.settings'];
  }

  public function getFormId() {
    return 'files_and_folders_settings_form';
  }

  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('files_and_folders.settings');

    // File System Settings
    $form['file_system_settings'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('File System Settings'),
      '#weight' => -10,
    ];

    $form['file_system_settings']['file_system'] = [
      '#type' => 'select',
      '#title' => $this->t('File System'),
      '#options' => [
        'public' => $this->t('Public'),
        'private' => $this->t('Private'),
      ],
      '#default_value' => $config->get('file_system') ?: 'public',
      '#required' => FALSE,
    ];

    // Folder Creation Settings
    $form['folder_creation'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Folder Creation'),
      '#tree' => TRUE,
    ];

    $form['folder_creation']['folders_wrapper'] = [
      '#type' => 'container',
      '#attributes' => ['id' => 'folders-fieldset-wrapper'],
    ];

    if (empty($form_state->get('num_folders'))) {
      $form_state->set('num_folders', 1);
    }

    for ($i = 0; $i < $form_state->get('num_folders'); $i++) {
      $form['folder_creation']['folders_wrapper']['folders'][$i] = $this->folderFields($i);
    }

    $form['folder_creation']['add_folder'] = [
      '#type' => 'submit',
      '#value' => $this->t('Add another folder'),
      '#submit' => ['::addOne'],
      '#ajax' => [
        'callback' => '::addmoreCallback',
        'wrapper' => 'folders-fieldset-wrapper',
      ],
    ];

    return parent::buildForm($form, $form_state);
  }

  private function folderFields($delta) {
    return [
      'name' => [
        '#type' => 'textfield',
        '#title' => $this->t('Folder Name'),
        '#required' => FALSE,
      ],
      'icon' => [
        '#type' => 'managed_file',
        '#title' => $this->t('Folder Icon'),
        '#upload_location' => 'public://folder-icons/',
        '#upload_validators' => [
          'file_validate_extensions' => ['png jpg jpeg svg'],
          'file_validate_is_image' => [],
          'file_validate_size' => [2 * 1024 * 1024], // 2MB limit
        ],
        '#description' => $this->t('Upload a custom icon for this folder. Allowed extensions: png, jpg, jpeg, svg. Max size: 2MB.'),
        '#theme' => 'image_widget',
        '#preview_image_style' => 'thumbnail',
      ],
      'authorized_roles' => [
        '#type' => 'checkboxes',
        '#title' => $this->t('Roles allowed to access folder'),
        '#options' => array_map(function ($role) {
          return $role->label();
        }, Role::loadMultiple()),
        '#description' => $this->t('Select which roles are allowed to access this folder.'),
      ],
    ];
  }

  public function addmoreCallback(array &$form, FormStateInterface $form_state) {
    return $form['folder_creation']['folders_wrapper'];
  }

  public function addOne(array &$form, FormStateInterface $form_state) {
    $num_folders = $form_state->get('num_folders');
    $form_state->set('num_folders', $num_folders + 1);
    $form_state->setRebuild();
  }

  public function validateForm(array &$form, FormStateInterface $form_state) {
    parent::validateForm($form, $form_state);
    
    $folders = $form_state->getValue(['folder_creation', 'folders_wrapper', 'folders']);
    foreach ($folders as $delta => $folder) {
      if (!empty($folder['icon'])) {
        $file = File::load($folder['icon'][0]);
        if ($file) {
          $image = \Drupal::service('image.factory')->get($file->getFileUri());
          if (!$image->isValid()) {
            $form_state->setError($form['folder_creation']['folders_wrapper']['folders'][$delta]['icon'], $this->t('The uploaded file is not a valid image.'));
          }
        }
      }
    }
  }

  public function submitForm(array &$form, FormStateInterface $form_state) {
    parent::submitForm($form, $form_state);
  
    $folders = $form_state->getValue(['folder_creation', 'folders_wrapper', 'folders']);
    
    if ($folders) {
      foreach ($folders as $folder) {
        $name = strtoupper($folder['name']);
        $authorized_roles = array_filter($folder['authorized_roles']);
      
        $node = Node::create([
          'type' => 'folder',
          'title' => $name,
          'field_authorized_roles' => array_keys($authorized_roles),
        ]);
  
        // Handle icon upload
        if (!empty($folder['icon'])) {
          $file = File::load($folder['icon'][0]);
          if ($file) {
            $file->setPermanent();
            $file->save();
            $node->set('field_folder_icon', [
              'target_id' => $file->id(),
              'alt' => $name . ' icon',
              'title' => $name . ' icon',
            ]);
          }
        }
  
        $node->save();
      }
      
      $this->messenger()->addMessage($this->t('Root folders created successfully.'));
    }
  }
}