<?php

namespace Drupal\file_folder_manager\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Ajax\CloseModalDialogCommand;
use Drupal\Core\Ajax\ReplaceCommand;
use Drupal\file_folder_manager\Controller\FileFolderController;
use Drupal\file\Entity\File;

class FolderForm extends FormBase {

  public function getFormId() {
    return 'file_folder_manager_folder_form';
  }

  public function buildForm(array $form, FormStateInterface $form_state) {
    $form['name'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Folder Name'),
      '#required' => TRUE,
    ];

    $form['parent'] = [
      '#type' => 'select',
      '#title' => $this->t('Parent Folder'),
      '#options' => $this->getFolderHierarchyOptions(),
      '#default_value' => 'root',
    ];

    $form['visibility'] = [
      '#type' => 'select',
      '#title' => $this->t('Visibility'),
      '#options' => [
        'public' => $this->t('Public'),
        'private' => $this->t('Private'),
        'shared' => $this->t('Shared'),
      ],
      '#required' => TRUE,
    ];

    $form['icon'] = [
      '#type' => 'managed_file',
      '#title' => $this->t('Folder Icon'),
      '#upload_location' => 'public://folder-icons/',
      '#upload_validators' => [
        'file_validate_extensions' => ['png jpg jpeg svg'],
        'file_validate_size' => [2 * 1024 * 1024], // 2MB limit
      ],
      '#description' => $this->t('Upload a custom icon for this folder. Allowed extensions: png, jpg, jpeg, svg. Max size: 2MB.'),
      '#states' => [
        'visible' => [
          ':input[name="parent"]' => ['value' => 'root'],
        ],
      ],
    ];

    $form['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Create Folder'),
      '#ajax' => [
        'callback' => '::submitFormAjax',
      ],
    ];

    return $form;
  }

  protected function getFolderHierarchyOptions() {
    // Implement a method to get the folder hierarchy as options
    // Return an array with 'root' and all folder IDs as keys, and their hierarchical names as values
    // Example: ['root' => 'Root', '1' => '- Public', '2' => '-- INFORMATIQUE', ...]
  }

  public function submitForm(array &$form, FormStateInterface $form_state) {
    $parent_id = $form_state->getValue('parent');
    $folder_data = [
      'type' => 'folder',
      'title' => $form_state->getValue('name'),
      'field_visibility' => $form_state->getValue('visibility'),
    ];

    if ($parent_id !== 'root') {
      $folder_data['field_parent'] = $parent_id;
    } else {
      // Handle icon upload for root folders
      if ($form_state->getValue('icon')) {
        $file = File::load($form_state->getValue('icon')[0]);
        if ($file) {
          $file->setPermanent();
          $file->save();
          $folder_data['field_folder_icon'] = [
            'target_id' => $file->id(),
            'alt' => $folder_data['title'] . ' icon',
            'title' => $folder_data['title'] . ' icon',
          ];
        }
      }
    }

    $folder = \Drupal\node\Entity\Node::create($folder_data);
    $folder->save();

    $this->messenger()->addMessage($this->t('Folder created successfully.'));
  }

  public function submitFormAjax(array &$form, FormStateInterface $form_state) {
    $response = new AjaxResponse();
    $response->addCommand(new CloseModalDialogCommand());
    $response->addCommand(new ReplaceCommand('.folder-hierarchy', FileFolderController::buildFolderTree()));
    return $response;
  }
}