<?php

namespace Drupal\files_and_folders\Service;

use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\Image\ImageFactory;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\file\FileInterface;
use Imagick;

/**
 * Service for generating thumbnails from various file types.
 */
class ThumbnailGenerator {

  /**
   * The file system service.
   *
   * @var \Drupal\Core\File\FileSystemInterface
   */
  protected $fileSystem;

  /**
   * The image factory.
   *
   * @var \Drupal\Core\Image\ImageFactory
   */
  protected $imageFactory;

  /**
   * The logger factory.
   *
   * @var \Drupal\Core\Logger\LoggerChannelFactoryInterface
   */
  protected $loggerFactory;

  /**
   * Constructs a new ThumbnailGenerator object.
   *
   * @param \Drupal\Core\File\FileSystemInterface $file_system
   *   The file system service.
   * @param \Drupal\Core\Image\ImageFactory $image_factory
   *   The image factory.
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $logger_factory
   *   The logger factory.
   */
  public function __construct(
    FileSystemInterface $file_system,
    ImageFactory $image_factory,
    LoggerChannelFactoryInterface $logger_factory
  ) {
    $this->fileSystem = $file_system;
    $this->imageFactory = $image_factory;
    $this->loggerFactory = $logger_factory->get('files_and_folders');
  }

  /**
   * Generates a thumbnail for the given file.
   *
   * @param \Drupal\file\FileInterface $file
   *   The file entity.
   *
   * @return string|null
   *   The URI of the generated thumbnail, or null if generation failed.
   */
  /**
   * Main generateThumbnail method update.
   */
  public function generateThumbnail(FileInterface $file) {
    $mime_type = $file->getMimeType();
    $extension = pathinfo($file->getFilename(), PATHINFO_EXTENSION);
  
    // Create thumbnails directory with unique naming
    $thumbnail_directory = 'public://thumbnails/' . date('Y-m');
    $this->fileSystem->prepareDirectory($thumbnail_directory, 
      FileSystemInterface::CREATE_DIRECTORY | 
      FileSystemInterface::MODIFY_PERMISSIONS
    );
  
    // Generate unique thumbnail filename using file ID and timestamp
    $thumbnail_uri = $thumbnail_directory . '/thumbnail_' . $file->id() . '_' . time() . '.jpg';
  
    // Log thumbnail generation attempt
    $this->loggerFactory->notice('Attempting to generate thumbnail for @type file: @name', [
      '@type' => $mime_type,
      '@name' => $file->getFilename()
    ]);
  
    try {
      // Handle different file types
      if ($mime_type === 'image/svg+xml') {
        return $this->generateSvgThumbnail($file, $thumbnail_uri);
      } 
      elseif (strpos($mime_type, 'image/') === 0) {
        return $this->generateImageThumbnail($file, $thumbnail_uri);
      } 
      elseif ($mime_type === 'application/pdf' || $extension === 'pdf') {
        return $this->generatePdfThumbnail($file, $thumbnail_uri);
      } 
      elseif (in_array($extension, ['txt'])) {
        return $this->generateTextThumbnail($file, $thumbnail_uri);
      } 
      elseif (in_array($extension, ['doc', 'docx', 'xls', 'xlsx', 'ppt', 'pptx'])) {
        return $this->generateOfficeThumbnail($file, $thumbnail_uri, $extension);
      }
    }
    catch (\Exception $e) {
      $this->loggerFactory->error('Thumbnail generation failed for file @fid: @message', [
        '@fid' => $file->id(),
        '@message' => $e->getMessage(),
      ]);
    }
  
    return null;
  }

  /**
   * Generates a thumbnail for a text file.
   */
  protected function generateTextThumbnail(FileInterface $file, string $thumbnail_uri) {
    try {
      if (!class_exists('Imagick')) {
        return null;
      }

      // Create Imagick instance
      $imagick = new \Imagick();
      
      // Create a new image with white background
      $imagick->newImage(300, 300, 'white');
      
      // Set up the drawing properties
      $draw = new \ImagickDraw();
      $draw->setFontSize(12);
      $draw->setFillColor('black');
      
      // Read first few lines of text file
      $content = file_get_contents($file->getFileUri(), FALSE, NULL, 0, 500);
      $lines = explode("\n", wordwrap($content, 40));
      
      // Only take first 15 lines
      $lines = array_slice($lines, 0, 15);
      
      // Add text to image
      $y = 30;
      foreach ($lines as $line) {
        $imagick->annotateImage($draw, 20, $y, 0, $line);
        $y += 18; // Line spacing
      }
      
      // Add file name at the bottom
      $draw->setFontSize(14);
      $draw->setFillColor('blue');
      $imagick->annotateImage($draw, 20, 280, 0, $file->getFilename());
      
      // Set format and quality
      $imagick->setImageFormat('jpeg');
      $imagick->setImageCompression(\Imagick::COMPRESSION_JPEG);
      $imagick->setImageCompressionQuality(90);
      
      // Save the thumbnail
      $imagick->writeImage($thumbnail_uri);
      $imagick->clear();
      $imagick->destroy();
      
      if (file_exists($thumbnail_uri)) {
        $this->loggerFactory->notice('Text file thumbnail created: @uri', [
          '@uri' => $thumbnail_uri
        ]);
        return $thumbnail_uri;
      }
    }
    catch (\Exception $e) {
      $this->loggerFactory->error('Error generating text file thumbnail: @message', [
        '@message' => $e->getMessage()
      ]);
    }
    
    return null;
  }
  
  /**
 * Generates a thumbnail for an image file.
 */
/**
 * Generates a thumbnail for an image file.
 */
protected function generateImageThumbnail(FileInterface $file, string $thumbnail_uri) {
  try {
    // Log the file information
    $this->loggerFactory->notice('Generating thumbnail for image: @mime, @uri', [
      '@mime' => $file->getMimeType(),
      '@uri' => $file->getFileUri()
    ]);

    if (class_exists('Imagick')) {
      $imagick = new \Imagick();
      $imagick->readImage($file->getFileUri());
      
      // Handle PNG transparency
      if ($file->getMimeType() === 'image/png') {
        $imagick->setImageBackgroundColor(new \ImagickPixel('white'));
        $imagick->setImageAlphaChannel(\Imagick::ALPHACHANNEL_REMOVE);
      }

      // Set high-quality settings
      $imagick->setImageCompressionQuality(90);
      
      // Get original dimensions
      $width = $imagick->getImageWidth();
      $height = $imagick->getImageHeight();
      
      // Calculate new dimensions maintaining aspect ratio
      $target_size = 300;
      if ($width > $height) {
        $new_width = $target_size;
        $new_height = intval($height * ($target_size / $width));
      } else {
        $new_height = $target_size;
        $new_width = intval($width * ($target_size / $height));
      }
      
      // Resize with high quality
      $imagick->resizeImage($new_width, $new_height, \Imagick::FILTER_LANCZOS, 1);
      
      // Convert to JPEG format
      $imagick->setImageFormat('jpeg');
      
      // Write the thumbnail
      $imagick->writeImage($thumbnail_uri);
      $imagick->clear();
      $imagick->destroy();
      
      if (file_exists($thumbnail_uri)) {
        $this->loggerFactory->notice('Image thumbnail created successfully: @uri', [
          '@uri' => $thumbnail_uri
        ]);
        return $thumbnail_uri;
      }
    } else {
      // Fallback to GD if Imagick is not available
      $image = $this->imageFactory->get($file->getFileUri());
      if (!$image->isValid()) {
        throw new \Exception('Invalid image file');
      }
      
      // Get original dimensions
      $width = $image->getWidth();
      $height = $image->getHeight();
      
      // Calculate new dimensions
      $target_size = 300;
      if ($width > $height) {
        $new_width = $target_size;
        $new_height = intval($height * ($target_size / $width));
      } else {
        $new_height = $target_size;
        $new_width = intval($width * ($target_size / $height));
      }
      
      // Create the thumbnail
      $image->scale($new_width, $new_height);
      $image->save($thumbnail_uri, ['quality' => 90]);
      
      if (file_exists($thumbnail_uri)) {
        $this->loggerFactory->notice('Image thumbnail created using GD: @uri', [
          '@uri' => $thumbnail_uri
        ]);
        return $thumbnail_uri;
      }
    }
  }
  catch (\Exception $e) {
    $this->loggerFactory->error('Error generating image thumbnail: @message', [
      '@message' => $e->getMessage()
    ]);
  }
  
  return null;
}

/**
 * Generates a thumbnail for a PDF file.
 */
protected function generatePdfThumbnail(FileInterface $file, string $thumbnail_uri) {
  try {
    if (!class_exists('Imagick')) {
      return null;
    }

    $imagick = new \Imagick();
    $imagick->setResolution(300, 300); // Higher resolution
    $imagick->readImage($file->getFileUri() . '[0]');
    
    // Convert to high-quality JPEG
    $imagick->setImageFormat('jpeg');
    $imagick->setImageCompression(\Imagick::COMPRESSION_JPEG);
    $imagick->setImageCompressionQuality(100);
    
    // Resize maintaining aspect ratio with high quality
    $imagick->resizeImage(1200, 1200, \Imagick::FILTER_LANCZOS, 1, true);
    
    // Set white background
    $imagick->setImageBackgroundColor('white');
    $imagick->setImageAlphaChannel(\Imagick::ALPHACHANNEL_REMOVE);
    
    // Center the content
    $imagick->trimImage(0);
    $imagick->setImagePage($imagick->getImageWidth(), $imagick->getImageHeight(), 0, 0);
    
    $imagick->writeImage($thumbnail_uri);
    $imagick->clear();
    $imagick->destroy();

    return $thumbnail_uri;
  }
  catch (\Exception $e) {
    $this->loggerFactory->error('Error generating PDF thumbnail: @message', [
      '@message' => $e->getMessage(),
    ]);
    return null;
  }
}
  
/**
   * Improved SVG thumbnail generation.
   */
  protected function generateSvgThumbnail(FileInterface $file, string $thumbnail_uri) {
    try {
      if (!class_exists('Imagick')) {
        return null;
      }

      $imagick = new \Imagick();
      $imagick->setBackgroundColor(new \ImagickPixel('transparent'));
      
      // Read SVG file
      $imagick->readImage($file->getFileUri());
      
      // Set resolution for better quality
      $imagick->setResolution(300, 300);
      
      // Convert to PNG first to handle transparency
      $imagick->setImageFormat('png');
      
      // Resize maintaining aspect ratio
      $imagick->resizeImage(300, 300, \Imagick::FILTER_LANCZOS, 1, true);
      
      // Convert to JPEG with white background
      $imagick->setImageFormat('jpeg');
      $imagick->setImageBackgroundColor('white');
      $imagick->setImageAlphaChannel(\Imagick::ALPHACHANNEL_REMOVE);
      
      // Set compression and quality
      $imagick->setImageCompression(\Imagick::COMPRESSION_JPEG);
      $imagick->setImageCompressionQuality(90);
      
      // Save the thumbnail
      $imagick->writeImage($thumbnail_uri);
      $imagick->clear();
      $imagick->destroy();
      
      if (file_exists($thumbnail_uri)) {
        $this->loggerFactory->notice('SVG thumbnail created: @uri', [
          '@uri' => $thumbnail_uri
        ]);
        return $thumbnail_uri;
      }
    }
    catch (\Exception $e) {
      $this->loggerFactory->error('Error generating SVG thumbnail: @message', [
        '@message' => $e->getMessage()
      ]);
    }
    
    return null;
  }

/**
 * Generates a thumbnail for Office documents.
 */
protected function generateOfficeThumbnail(FileInterface $file, string $thumbnail_uri, string $extension) {
  try {
    // Create temporary directory for conversion
    $tempDir = \Drupal::service('file_system')->getTempDirectory() . '/office_thumb_' . uniqid();
    \Drupal::service('file_system')->prepareDirectory($tempDir, FileSystemInterface::CREATE_DIRECTORY);
    
    // Get real path of the file
    $filePath = \Drupal::service('file_system')->realpath($file->getFileUri());
    
    if (!$filePath || !file_exists($filePath)) {
      throw new \Exception('Office document file not found at: ' . $filePath);
    }

    // Convert Office document to PDF using LibreOffice
    $command = sprintf(
      'soffice --headless --convert-to pdf --outdir %s %s',
      escapeshellarg($tempDir),
      escapeshellarg($filePath)
    );
    
    exec($command, $output, $return_var);

    if ($return_var !== 0) {
      throw new \Exception('Failed to convert office document to PDF');
    }

    // Get the generated PDF file
    $pdfFiles = glob($tempDir . '/*.pdf');
    if (empty($pdfFiles)) {
      throw new \Exception('No PDF file generated in: ' . $tempDir);
    }

    // Create Imagick instance
    $imagick = new \Imagick();
    
    // Set high resolution for better quality
    $imagick->setResolution(300, 300);
    
    // Read the first page of the converted PDF
    $imagick->readImage($pdfFiles[0] . '[0]');
    
    // Convert to high-quality JPEG
    $imagick->setImageFormat('jpeg');
    $imagick->setImageCompression(\Imagick::COMPRESSION_JPEG);
    $imagick->setImageCompressionQuality(100);
    
    // Resize maintaining aspect ratio with high quality
    $imagick->resizeImage(1200, 1200, \Imagick::FILTER_LANCZOS, 1, true);
    
    // Set white background
    $imagick->setImageBackgroundColor('white');
    $imagick->setImageAlphaChannel(\Imagick::ALPHACHANNEL_REMOVE);
    
    // Center the content
    $imagick->trimImage(0);
    $imagick->setImagePage($imagick->getImageWidth(), $imagick->getImageHeight(), 0, 0);
    
    // Save the thumbnail
    $imagick->writeImage($thumbnail_uri);
    $imagick->clear();
    $imagick->destroy();

    // Clean up temporary files
    \Drupal::service('file_system')->deleteRecursive($tempDir);

    if (file_exists($thumbnail_uri)) {
      $this->loggerFactory->notice('Office document thumbnail created: @uri', [
        '@uri' => $thumbnail_uri
      ]);
      return $thumbnail_uri;
    }

    throw new \Exception('Failed to create thumbnail');
  }
  catch (\Exception $e) {
    $this->loggerFactory->error('Error generating office document thumbnail: @message', [
      '@message' => $e->getMessage()
    ]);
    
    // Fallback to default icons if thumbnail generation fails
    return $this->getDefaultIconPath($extension);
  }
}

  /**
   * Returns the path to a default icon based on file extension.
   */
  protected function getDefaultIconPath($extension) {
    $icons = [
      'doc' => 'word',
      'docx' => 'word',
      'xls' => 'excel',
      'xlsx' => 'excel',
      'ppt' => 'powerpoint',
      'pptx' => 'powerpoint',
    ];
    
    $icon_type = $icons[$extension] ?? 'document';
    return drupal_get_path('module', 'files_and_folders') . '/icons/' . $icon_type . '.png';
  }
}